@@ -1,14 +1,13 @@
 package Perinci::CmdLine::Inline;
 
-our $DATE = '2017-01-12'; # DATE
-our $VERSION = '0.47'; # VERSION
+our $DATE = '2017-01-16'; # DATE
+our $VERSION = '0.48'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
 use Log::Any::IfLOG qw($log);
 
-use Data::Clean::JSON;
 use Data::Dmp;
 use JSON::MaybeXS ();
 use Module::CoreList::More;
@@ -87,6 +86,27 @@ sub _gen_read_env {
     join("", @l2);
 }
 
+sub _gen_enable_log {
+    my ($cd) = @_;
+
+    _add_module($cd, 'Log::Any::Adapter::Screen');
+    _add_module($cd, 'Log::Any::Adapter'); # required by Log::Any::Adapter::Screen
+    _add_module($cd, 'Log::Any'); # required by Log::Any::Adapter
+    _add_module($cd, 'Log::Any::Manager'); # required by Log::Any
+    _add_module($cd, 'Log::Any::Proxy::Null'); # required by Log::Any
+    _add_module($cd, 'Log::Any::Adapter::Util'); # required by Log::Any
+
+    my @l;
+
+    push @l, "### enable logging\n";
+    push @l, 'require Log::Any; my $log = Log::Any->get_logger;', "\n";
+    push @l, 'require Log::Any::Adapter;', "\n";
+    push @l, 'Log::Any::Adapter->set("Screen");', "\n";
+    push @l, "\n";
+
+    join("", @l);
+}
+
 sub _gen_read_config {
     my ($cd) = @_;
     my @l2;
@@ -100,6 +120,7 @@ sub _gen_read_config {
     _add_module($cd, "Data::Sah::Normalize"); # required by Perinci::CmdLine::Util::Config
     _add_module($cd, "Perinci::Sub::Normalize"); # required by Perinci::CmdLine::Util::Config
     _add_module($cd, "Sah::Schema::rinci::function_meta"); # required by Perinci::Sub::Normalize
+    push @l2, '$log->tracef("Reading config file(s) ...");', "\n" if $cd->{gen_args}{log};
     push @l2, '  require Perinci::CmdLine::Util::Config;', "\n";
     push @l2, "\n";
     push @l2, '  my $res = Perinci::CmdLine::Util::Config::read_config(', "\n";
@@ -404,16 +425,40 @@ sub _gen_common_opt_handler {
         my $mod = $cd->{sc_mods}{''};
         push @l, "no warnings 'once'; ";
         push @l, "require $mod; " if $mod;
-        push @l, 'print "', $cd->{script_name} , ' version ", ',
-            (defined($cd->{gen_args}{script_version}) ? "\"$cd->{gen_args}{script_version}\"" :
-                 "(\$$mod\::VERSION // '?')"),
-                     ", (\$$mod\::DATE ? \" (\$$mod\::DATE)\" : '')",
-                         ', "\\n"; ';
+        push @l, 'print "', $cd->{script_name} , ' version ", ';
+        if ($cd->{gen_args}{script_version_from_main_version}) {
+            push @l, "\$main::VERSION // '?'", ", (\$main::DATE ? \" (\$main\::DATE)\" : '')";
+        } else {
+            push @l, defined($cd->{gen_args}{script_version}) ? "\"$cd->{gen_args}{script_version}\"" :
+                "(\$$mod\::VERSION // '?')",
+                    ", (\$$mod\::DATE ? \" (\$$mod\::DATE)\" : '')";
+        }
+        push @l, ', "\\n"; ';
         push @l, 'print "  Generated by ', __PACKAGE__ , ' version ',
             (${__PACKAGE__."::VERSION"} // 'dev'),
                 (${__PACKAGE__."::DATE"} ? " (".${__PACKAGE__."::DATE"}.")" : ""),
                     '\n"; ';
         push @l, 'exit 0';
+    } elsif ($co eq 'log_level') {
+        push @l, 'if ($_[1] eq "trace") { Log::Any::Adapter->set("Screen", min_level=>"trace") } ';
+        push @l, 'if ($_[1] eq "debug") { Log::Any::Adapter->set("Screen", min_level=>"debug") } ';
+        push @l, 'if ($_[1] eq "info" ) { Log::Any::Adapter->set("Screen", min_level=>"info" ) } ';
+        push @l, 'if ($_[1] eq "error") { Log::Any::Adapter->set("Screen", min_level=>"error") } ';
+        push @l, 'if ($_[1] eq "fatal") { Log::Any::Adapter->set("Screen", min_level=>"fatal") } ';
+        push @l, 'if ($_[1] eq "none")  { Log::Any::Adapter->set("Screen", min_level=>"none")  } ';
+        push @l, '$_pci_r->{log_level} = $_[1];';
+    } elsif ($co eq 'trace') {
+        push @l, 'Log::Any::Adapter->set("Screen", min_level=>"trace"); ';
+        push @l, '$_pci_r->{log_level} = "trace";';
+    } elsif ($co eq 'debug') {
+        push @l, 'Log::Any::Adapter->set("Screen", min_level=>"debug"); ';
+        push @l, '$_pci_r->{log_level} = "debug";';
+    } elsif ($co eq 'verbose') {
+        push @l, 'Log::Any::Adapter->set("Screen", min_level=>"info"); ';
+        push @l, '$_pci_r->{log_level} = "info";';
+    } elsif ($co eq 'quiet') {
+        push @l, 'Log::Any::Adapter->set("Screen", min_level=>"error"); ';
+        push @l, '$_pci_r->{log_level} = "error";';
     } elsif ($co eq 'subcommands') {
         my $scs_text = "Available subcommands:\n";
         for (sort keys %{ $cd->{metas} }) {
@@ -432,14 +477,14 @@ sub _gen_common_opt_handler {
     } elsif ($co eq 'no_naked_res') {
         push @l, '$_pci_r->{naked_res} = 0;';
     } elsif ($co eq 'no_config') {
-        push @l, '        $_pci_r->{read_config} = 0;';
+        push @l, '$_pci_r->{read_config} = 0;';
     } elsif ($co eq 'config_path') {
-        push @l, '        $_pci_r->{config_paths} //= [];';
-        push @l, '        push @{ $_pci_r->{config_paths} }, $_[1];';
+        push @l, '$_pci_r->{config_paths} //= []; ';
+        push @l, 'push @{ $_pci_r->{config_paths} }, $_[1];';
     } elsif ($co eq 'config_profile') {
-        push @l, '        $_pci_r->{config_profile} = $_[1];';
+        push @l, '$_pci_r->{config_profile} = $_[1];';
     } elsif ($co eq 'no_env') {
-        push @l, '        $_pci_r->{read_env} = 0;';
+        push @l, '$_pci_r->{read_env} = 0;';
     } else {
         die "BUG: Unrecognized common_opt '$co'";
     }
@@ -455,6 +500,7 @@ sub _gen_get_args {
 
     _add_module($cd, "Getopt::Long::EvenLess");
     push @l, "require Getopt::Long::EvenLess;\n";
+    push @l, '$log->tracef("Parsing command-line arguments ...");', "\n" if $cd->{gen_args}{log};
 
     if ($cd->{gen_args}{subcommands}) {
 
@@ -666,7 +712,6 @@ my %pericmd_attrs = (
           default_format
           description exit formats
           riap_client riap_version riap_client_args
-          log
           tags
           get_subcommand_from_arg
          /),
@@ -686,6 +731,10 @@ my %pericmd_attrs = (
         summary => 'Script version (otherwise will use version from url metadata)',
         schema => 'str',
     },
+    script_version_from_main_version => {
+        summary => "Use script's \$main::VERSION for the version",
+        schema => 'bool*',
+    },
     url => {
         summary => 'Program URL',
         schema => 'riap::url*',
@@ -712,6 +761,21 @@ _
         schema  => 'bool*',
         default => 0,
     },
+    use_cleanser => {
+        summary => 'Whether to use data cleanser routine first before producing JSON',
+        schema => 'bool*',
+        default => 1,
+        description => <<'_',
+
+When a function returns result, and the user wants to display the result as
+JSON, the result might need to be cleansed first (e.g. using <pm:Data::Clean>)
+before it can be encoded to JSON, for example it might contain Perl objects or
+scalar references or other stuffs. If you are sure that your function does not
+produce those kinds of data, you can set this to false to produce a more
+lightweight script.
+
+_
+    },
 );
 
 $SPEC{gen_inline_pericmd_script} = {
@@ -830,6 +894,11 @@ _
             summary => 'Name of environment variable name that sets default options',
             schema => 'str*',
         },
+        log => {
+            summary => 'Whether to enable logging',
+            schema  => 'bool*',
+            default => 0,
+        },
 
         with_debug => {
             summary => 'Generate script with debugging outputs',
@@ -915,6 +984,7 @@ sub gen_inline_pericmd_script {
     $args{pack_deps} //= 1;
     $args{read_config} //= 1;
     $args{read_env} //= 1;
+    $args{use_cleanser} //= 1;
 
     my $cd = {
         gen_args => \%args,
@@ -1038,8 +1108,29 @@ sub gen_inline_pericmd_script {
             my %copts;
             $copts{help} = $Perinci::CmdLine::Base::copts{help};
             $copts{version} = $Perinci::CmdLine::Base::copts{version};
+            if ($args{log}) {
+                $copts{log_level} = {
+                    getopt  => "log-level=s",
+                    summary => "Set logging level (trace|debug|info|warn|error|fatal|none)",
+                };
+                $copts{trace} = {
+                    getopt  => "trace",
+                    summary => "Set logging level to trace",
+                };
+                $copts{debug} = {
+                    getopt  => "debug",
+                    summary => "Set logging level to debug",
+                };
+                $copts{verbose} = {
+                    getopt  => "verbose",
+                    summary => "Set logging level to info",
+                };
+                $copts{quiet} = {
+                    getopt  => "quiet",
+                    summary => "Set logging level to error",
+                };
+            }
             unless ($args{skip_format}) {
-                _add_module($cd, "JSON::Tiny::Subclassable");
                 $copts{json} = $Perinci::CmdLine::Base::copts{json};
                 $copts{format} = $Perinci::CmdLine::Base::copts{format};
                 # "naked_res!" currently not supported by
@@ -1133,17 +1224,30 @@ _
 
         $cd->{sub_srcs}{_pci_json} = <<'_';
     state $json = do {
-        if    (eval { require JSON::XS; 1 }) { JSON::XS->new->canonical(1)->allow_nonref }
-        elsif (eval { require JSON::PP; 1 }) { JSON::PP->new->canonical(1)->allow_nonref }
-        else { require JSON::Tiny::Subclassable; JSON::Tiny::Subclassable->new }
+        if (eval { require JSON::XS; 1 }) { JSON::XS->new->canonical(1)->allow_nonref }
+        else { require JSON::PP; JSON::PP->new->canonical(1)->allow_nonref }
     };
     $json;
 _
 
         {
-            my $cleanser = Data::Clean::JSON->get_cleanser;
-            my $src = $cleanser->{src};
-            $cd->{module_srcs}{'Local::_pci_clean_json'} = "require Scalar::Util; use feature 'state'; sub _pci_clean_json { state \$cleanser = $src; \$cleanser->(shift) }\n1;\n";
+            last unless $args{use_cleanser};
+            require Module::CoreList;
+            require Data::Clean::JSON;
+            my $cleanser = Data::Clean::JSON->new(
+                # pick this noncore PP module instead of the default non-core XS
+                # module Data::Clone. perl has core module Storable, but
+                # Storable still chooses to croak on Regexp objects.
+                '!clone_func' => 'Clone::PP::clone',
+            );
+            my $src = $cleanser->{_cd}{src};
+            my $src1 = 'sub _pci_clean_json { ';
+            for my $mod (keys %{ $cleanser->{_cd}{modules} }) {
+                $src1 .= "require $mod; ";
+                next if Module::CoreList->is_core($mod);
+                _add_module($cd, $mod);
+            }
+            $cd->{module_srcs}{'Local::_pci_clean_json'} = "$src1 use feature 'state'; state \$cleanser = $src; \$cleanser->(shift) }\n1;\n";
         }
 
         {
@@ -1202,6 +1306,7 @@ _
         $cd->{vars}{'$_pci_meta_result_type'} = undef;
         $cd->{vars}{'$_pci_meta_result_type_is_simple'} = undef;
         push @l, "{\n";
+        push @l, '$log->tracef("Calling function ...");', "\n" if $cd->{gen_args}{log};
         push @l, 'my $sc_name = $_pci_r->{subcommand_name};' . "\n";
         push @l, '$_pci_args{-cmdline} = Perinci::CmdLine::Inline::Object->new(@{', dmp([%args]), '});', "\n"
             if $args{pass_cmdline_object};
@@ -1229,6 +1334,7 @@ _
         # generate code to format & display result
         push @l, "### format & display result\n\n";
         push @l, "{\n";
+        push @l, '$log->tracef("Displaying result ...");', "\n" if $cd->{gen_args}{log};
         push @l, 'my $fres;', "\n";
         push @l, 'my $save_res; if (exists $_pci_r->{res}[3]{"cmdline.result"}) { $save_res = $_pci_r->{res}[2]; $_pci_r->{res}[2] = $_pci_r->{res}[3]{"cmdline.result"} }', "\n";
         push @l, 'my $is_success = $_pci_r->{res}[0] =~ /\A2/ || $_pci_r->{res}[0] == 304;', "\n";
@@ -1236,8 +1342,10 @@ _
         push @l, 'if ($is_success && (', ($args{skip_format} ? 1:0), ' || $_pci_meta_skip_format || $_pci_r->{res}[3]{"cmdline.skip_format"})) { $fres = $_pci_r->{res}[2] }', "\n";
         push @l, 'elsif ($is_success && $is_stream) {}', "\n";
         push @l, 'else { ';
-        push @l, 'require Local::_pci_clean_json; ' if $args{pack_deps};
-        push @l, 'require Perinci::Result::Format::Lite; $is_stream=0; _pci_clean_json($_pci_r->{res}); $fres = Perinci::Result::Format::Lite::format($_pci_r->{res}, ($_pci_r->{format} // $_pci_r->{res}[3]{"cmdline.default_format"} // "text"), $_pci_r->{naked_res}, 0) }', "\n";
+        push @l, 'require Local::_pci_clean_json; ' if $args{pack_deps} && $args{use_cleanser};
+        push @l, 'require Perinci::Result::Format::Lite; $is_stream=0; ';
+        push @l, '_pci_clean_json($_pci_r->{res}); ' if $args{use_cleanser};
+        push @l, '$fres = Perinci::Result::Format::Lite::format($_pci_r->{res}, ($_pci_r->{format} // $_pci_r->{res}[3]{"cmdline.default_format"} // "text"), $_pci_r->{naked_res}, 0) }', "\n";
         push @l, "\n";
 
         push @l, 'my $use_utf8 = $_pci_r->{res}[3]{"x.hint.result_binary"} ? 0 : '.($args{use_utf8} ? 1:0).";\n";
@@ -1335,7 +1443,7 @@ _
                 for (grep {/^code_/} keys %tmp) {
                     $tmp{$_} = "...";
                 }
-                JSON::MaybeXS::encode_json(\%tmp);
+                JSON::MaybeXS->new->canonical(1)->encode(\%tmp);
             }, "\n\n",
 
             'my $_pci_metas = ', do {
@@ -1374,6 +1482,8 @@ _
             (map {"require $_;\n"} sort keys %{$cd->{req_modules}}),
             "\n",
 
+            $args{log} ? _gen_enable_log() : '',
+
             "### declare global variables\n\n",
             (map { "our $_" . (defined($cd->{vars}{$_}) ? " = ".dmp($cd->{vars}{$_}) : "").";\n" } sort keys %{$cd->{vars}}),
             (keys(%{$cd->{vars}}) ? "\n" : ""),
@@ -1445,7 +1555,7 @@ Perinci::CmdLine::Inline - Generate inline Perinci::CmdLine CLI script
 
 =head1 VERSION
 
-This document describes version 0.47 of Perinci::CmdLine::Inline (from Perl distribution Perinci-CmdLine-Inline), released on 2017-01-12.
+This document describes version 0.48 of Perinci::CmdLine::Inline (from Perl distribution Perinci-CmdLine-Inline), released on 2017-01-16.
 
 =head1 SYNOPSIS
 
@@ -1608,9 +1718,9 @@ Currently does nothing, provided only for compatibility with Perinci::CmdLine::B
 
 Include extra modules.
 
-=item * B<log> => I<any>
+=item * B<log> => I<bool> (default: 0)
 
-Currently does nothing, provided only for compatibility with Perinci::CmdLine::Base.
+Whether to enable logging.
 
 =item * B<meta> => I<hash>
 
@@ -1675,6 +1785,10 @@ Currently does nothing, provided only for compatibility with Perinci::CmdLine::B
 
 Script version (otherwise will use version from url metadata).
 
+=item * B<script_version_from_main_version> => I<bool>
+
+Use script's $main::VERSION for the version.
+
 =item * B<shebang> => I<str>
 
 Set shebang line.
@@ -1695,6 +1809,17 @@ Currently does nothing, provided only for compatibility with Perinci::CmdLine::B
 
 Program URL.
 
+=item * B<use_cleanser> => I<bool> (default: 1)
+
+Whether to use data cleanser routine first before producing JSON.
+
+When a function returns result, and the user wants to display the result as
+JSON, the result might need to be cleansed first (e.g. using L<Data::Clean>)
+before it can be encoded to JSON, for example it might contain Perl objects or
+scalar references or other stuffs. If you are sure that your function does not
+produce those kinds of data, you can set this to false to produce a more
+lightweight script.
+
 =item * B<use_utf8> => I<bool> (default: 0)
 
 Whether to set utf8 flag on output.
